/*
* Copyright (C) 2013, Thomas E Latham, Matthew M Reid
*
* $Id: TreeWriter.hpp,v1.1 2013-07-28  mreid Exp $
* 
* This file is part of CharmlessSelection package.
*
*/

#ifndef TREEWRITER_HPP
#define TREEWRITER_HPP 1

// STL include
#include <iosfwd>
#include <iostream>
#include <string>
#include <map>
#include <vector>
#include <exception>
#include <utility>

// boost include
//#ifndef __CINT__
  // place boost headers
  #include "boost/any.hpp"
  //using namespace boost::boost::any;
  //using namespace boost::boost::any_cast;
//#else 
  // Forward declare
  //class boost;
  //class boost::any;
  //class boost::any_cast;
  //class boost::bad_boost::any_cast;
//#endif

// ROOT include
#include "TFile.h"
#include "TTree.h"
#include "Rtypes.h"

/*!
  A class for handling error from TreeWriter.
*/ 
class TreeWriterException: public std::exception {

  public:

    TreeWriterException(const std::string& msg):
      _Str("TreeWriterException: ") {
      _Str += msg;
    }
  
    virtual ~TreeWriterException(void) throw() { }
    virtual const char *what() const throw() { return _Str.c_str(); }

  private:

    std::string _Str;

};


/*!
  TreeWriter class can be used as an interface to TTree class and 
  conveniently writes data to a root TFile in a simple way.
*/ 
class TreeWriter {
  
  public:

    TreeWriter();
    TreeWriter( const std::string& filename, TTree* tree );
    TreeWriter( const std::string& filename, const std::string& treename );

    virtual ~TreeWriter();

    //! Set the column values
    /*!
      \param variable - the name of the variable.
      \param value - the value of the variable.
    */
    void column( const std::string& variable, const boost::any& value ) throw( TreeWriterException ); 

    //! Write columns to TTree
    void write( ) throw( TreeWriterException ); 
    
    //! Write Column values for given tree entry
    /*!
      \param entry - value of TTree entry.
    */
    void write( const unsigned int& entry ) throw( TreeWriterException ); 

    //! Set the tree name via std::string
    void setTreeName( const std::string& entry ); 
    //! Set the tree name
    void setTreeName( const char* entry ); 

    //! Get the Tree name
    std::string getTreeName() const { return m_tree->GetName(); }

    //! Get list of column names
    std::vector<std::string> getNames() const { return m_variables; }

  private: 

    void check_variable( const std::string& variable, const boost::any& value );

    bool is_empty( const boost::any& operand );
    bool is_int( const boost::any& operand );
    bool is_float( const boost::any& operand );
    bool is_double( const boost::any& operand );
    bool is_bool( const boost::any& operand );
 
    ClassDef(TreeWriter,1); // TreeWriter Class
      		
  protected:
    bool m_isFirst;
    size_t m_counter;
    TFile* m_file;
    TTree* m_tree;
    TTree* m_temp;
    std::vector< std::string > m_variables;
    std::map<std::string, Int_t > m_int_variables;
    std::map<std::string, Float_t > m_float_variables;
    std::map<std::string, Double_t > m_double_variables;
    std::map<std::string, Bool_t > m_bool_variables;
    //const Series::EODSeries& m_series; 
};

#endif // TREEWRITER_HPP
